#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>
#include <queue>
#include <cmath>
using namespace std; typedef long long ll; typedef long double ld;

ld const inf=1e60L;
int n; // number of vertices

struct Edge{
  int s,t;
  ll cap;
  ld cost;
  Edge *rev;
  static Edge E[];
  static int N;
};
Edge Edge::E[500*500*2];
int Edge::N=0;

vector<Edge*> edge[500*500*2]; // need 2*edges due to reverses
bool          done[500*500*2];
Edge*         from[500*500*2];
ld            dist[500*500*2], pot[500*500*2];

void add_edge(int s,int t,ll cap,ld cost){
  Edge::E[Edge::N]={s,t,cap,cost,Edge::E+Edge::N+1};
  edge[s].push_back(Edge::E+Edge::N++);

  Edge::E[Edge::N]={t,s, 0,-cost,Edge::E+Edge::N-1};
  edge[t].push_back(Edge::E+Edge::N++);
}

ld augment(){
  /* Reset accumulators */
  for (int i=n; i--;) done[i]=from[i]=0, dist[i]=inf;

  /* Dijkstra's algorithm */
  priority_queue<pair<ll,int>> todo;
  for (todo.push(make_pair(dist[0]=0,0)); not todo.empty();){
    int x=todo.top().second; todo.pop();
    if (done[x]) continue; else done[x]=true;
    for (auto& y: edge[x])
      if (y->cap)
        for (ld d=dist[x]+y->cost+pot[x]-pot[y->t]; d+1e-10L<dist[y->t] and not done[y->t]; ++d)
          todo.push(make_pair(-d,y->t)),
          from[y->t]=y,
          dist[y->t]=d;
  }

  /* Update vertex potentials */
  for (int i=n; i--;) pot[i]=min(inf,pot[i]+dist[i]);
  if (not from[1]){
    cerr<<"failed to augment"<<endl;
    return inf;
  }

  /* Propagate augmentation */
  ld cost=0;
  for (int x=1; x; x=from[x]->s) cost+=from[x]->cost, from[x]->cap--, from[x]->rev->cap++;
  return cost;
}

ld mcmf(int reps){
  ld res=0;
  for (int i=n; i--;) pot[i]=0;
  while (reps-->0) res+=augment();
  return res;
}

int main(){
  // reading the input - nb. n is global here
  ll sn,tn,q;
  cin>>n>>sn>>tn>>q;
  static ll hill[500][3];
  for (int i=0; i<n; i++)
    for (int j=0; j<3; j++)
      cin>>hill[i][j];

  // edges between hills
  for (int i=0; i<n; i++)
    for (int j=0; j<n; j++)
      if (hill[j][2]<hill[i][2] and
            (hill[j][0]-hill[i][0])*(hill[j][0]-hill[i][0])+
            (hill[j][1]-hill[i][1])*(hill[j][1]-hill[i][1])+
            (hill[j][2]-hill[i][2])*(hill[j][2]-hill[i][2])<=q*q){
        add_edge(i+2,j+2,sn+tn,hypotl(hypotl(hill[j][0]-hill[i][0],hill[j][1]-hill[i][1]),hill[j][2]-hill[i][2]));
      }

  // edges to/from sources and sinks
  for (int i=0; i<sn; i++) {int x; cin>>x; ++x; add_edge(0,x,1,0);}
  for (int i=0; i<tn; i++) {int x; cin>>x; ++x; add_edge(x,1,1,0);}

  // add in extra src/sink vertices
  n+=2;

  // solve!
  ld cost=mcmf(tn);
  if (cost<inf){
    cout.precision(8);
    cout<<fixed<<cost<<endl;
  }else{
    cout<<"IMPOSSIBLE\n";
  }
}
