#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>
#include <cmath>
#include <map>
#include <set>
using namespace std;
typedef long double ld;
typedef long long ll;
ld const oo=1e90L;

struct Hill{
    ll x,y,z;
    int id;
    bool operator<(Hill const &a)const{
        return z^a.z? z<a.z :
               x^a.z? x<a.x :
               y^a.y? y<a.y :
               id^a.id? id<a.id :
               false;
    }
};

ll sqdist(Hill const &a, Hill const &b){
    return (a.x-b.x)*(a.x-b.x) + (a.y-b.y)*(a.y-b.y) + (a.z-b.z)*(a.z-b.z);
}

int main(){
    ios::sync_with_stdio(false);
    ll n,sn,tn,q; cin>>n>>sn>>tn>>q;

    // Eliminate impossible case so we can guarantee t<=s
    if (tn>sn){
        cout<<"IMPOSSIBLE\n";
        return 0;
    }

    // Read in hills and sort by height
    vector<Hill> hills(n);
    for (int i=0; i<n; i++){
        cin>>hills[i].x>>hills[i].y>>hills[i].z;
        hills[i].id=i+1;
    }
    sort(hills.begin(),hills.end());

    // Read in sources and sinks
    vector<int> s(sn),t(tn);
    for (auto &i: s) cin>>i; sort(s.begin(),s.end());
    for (auto &i: t) cin>>i; sort(t.begin(),t.end());


    // Prepare cost matrix
    vector<vector<ld>> cost(sn,vector<ld>(tn,oo));

    // Work out distances
    for (int i=n; i--;){
        if (not binary_search(s.begin(),s.end(),hills[i].id)) continue;
        vector<ld> dist(n,oo);
        dist[i]=0.L;
        for (int j=i+1; j--;) if (dist[j]<oo) {
            for (int k=j; k--;) if (hills[k].z<hills[j].z) {
                ll travel = sqdist(hills[j],hills[k]);
                if (travel <= q*q) dist[k]=min(dist[k],dist[j]+sqrtl(travel));
            }
        }
        for (int j=n; j--;){
            if (not binary_search(t.begin(),t.end(),hills[j].id)) continue;
            int id=lower_bound(s.begin(),s.end(),hills[i].id)-s.begin();
            int jd=lower_bound(t.begin(),t.end(),hills[j].id)-t.begin();
            cost[id][jd]=dist[j];
            // if (dist[j]<oo) cerr<<hills[i].id<<" "<<hills[j].id<<" "<<dist[j]<<endl;
        }
    }

    // Dynamic programming to find minimum cost matching
    static ld res[41][1<<21];
    for (int i=1<<tn; i--;) for (int j=sn+1; j--;) res[j][i]=oo;
    res[0][0]=0.L;
    for (int i=0; i<sn; i++){
        for (ll mask=0; mask<(1ll<<(ll)tn); ++mask){
            // Use nothing
            res[i+1][mask]=min(res[i+1][mask],res[i][mask]);

            // Use something
            for (ll j=0; j<tn; j++){
                res[i+1][mask|(1ll<<j)]=min(res[i+1][mask|(1ll<<j)],res[i][mask]+cost[i][j]);
            }
        }
    }

    // Publish our results
    cout.precision(10);
    if (res[sn][(1<<tn)-1]<oo)
        cout<<fixed<<res[sn][(1<<tn)-1]<<endl;
    else
        cout<<"IMPOSSIBLE"<<endl;
}
