#include <algorithm>
#include <iostream>
#include <cassert>
#include <cstdint>
#include <vector>
#include <queue>
#include <cmath>
#include <map>
using namespace std;
typedef long long ll;
typedef long double ld;

struct Hill{
    ll x,y,z;
};

map<int,ld> edge[81];

map<int,ld> dijkstra(vector<Hill> const& hills,ll q,int src){
    map<int,ld> res;
    priority_queue<pair<ld,int>> todo;
    todo.push({0.L,src});
    while (not todo.empty()){
        int x=todo.top().second; ld dist=-todo.top().first; todo.pop();
        if (res.count(x)) continue;
        res[x]=dist;
        for (int i=hills.size(); i--;)
            if (hills[i].z<hills[x].z and
                (hills[i].x-hills[x].x)*(ll)(hills[i].x-hills[x].x)
               +(hills[i].y-hills[x].y)*(ll)(hills[i].y-hills[x].y)
               +(hills[i].z-hills[x].z)*(ll)(hills[i].z-hills[x].z)<=q*q){
                todo.push({-(dist+hypotl(hypotl(
                    hills[i].x-hills[x].x,
                    hills[i].y-hills[x].y),
                    hills[i].z-hills[x].z)), i});
            }
    }
    return res;
}

ld bellmanford(int s,int t){
    vector<pair<ld,int>> dist(81,{1e60L,-1});
    dist[s]={0L,s};
    for (int z=81; z--;)
        for (int i=81; i--;)
            if (~dist[i].second)
                for (auto j: edge[i])
                    dist[j.first]=min(dist[j.first],{dist[i].first+j.second,i});
    for (int i=t; dist[i].second!=-1 and i!=s; i=dist[i].second)
        edge[i][dist[i].second]=edge[dist[i].second][i],
        edge[dist[i].second].erase(i);
    return dist[t].first;
}

int identify(int x){
    static map<int,int> memo;
    static int cid=0;
    if (not memo[x]) memo[x]=++cid;
    return memo[x]-1;
}

int main(){
    int n,s,t,q; cin>>n>>s>>t>>q;

    vector<Hill> hills(n);
    for (auto &i: hills) cin>>i.x>>i.y>>i.z;

    vector<int> springs(s),towns(t);
    for (auto &i: springs) cin>>i, --i;
    for (auto &i: towns) cin>>i, --i;
    sort(springs.begin(),springs.end());
    sort(towns.begin(),towns.end());

    int const SINK=80;
    for (auto i: springs){
        map<int,ld> distances=dijkstra(hills,q,i);
        for (auto j: towns){
            if (distances.count(j)){
                edge[identify(~j)][identify(i)]=distances[j];
                edge              [identify(i)][SINK]=0;
                // cerr<<i+1<<" "<<j+1<<" "<<distances[j]<<endl;
            }
        }
    }

    ld res=0.L;
    for (auto i: towns)
        res+=bellmanford(identify(~i),SINK);

    if (res>=1e60L){
        cout<<"IMPOSSIBLE\n";
    }else{
        cout.precision(6);
        cout<<fixed<<res<<endl;
    }
}
