#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <map>
//#include <vector>
#include <cstdio>

#include "libchecktestdata.hpp"

using namespace std;
using namespace checktestdata;

ifstream in, ans;
istringstream team, prog;
string feedbackdir;

const string output_grammar = "\
REGEX(\"[A-Za-z]{1,20}\",first)\n\
IF( first==\"IMPOSSIBLE\" )\n\
	IF( !ISEOF ) NEWLINE END\n\
ELSE\n\
	SPACE INT(0,10^9) NEWLINE\n\
	WHILE( !ISEOF )\n\
		REGEX(\"[A-Za-z]{1,20}\") SPACE INT(0,10^9)\n\
		IF( !ISEOF ) NEWLINE END\n\
	END\n\
END\n\
WHILE( !ISEOF ) NEWLINE END\n\
";

// Internal error, should not happen
void error(string msg)
{
	cerr << "Error: " << msg << endl;
	exit(1);
}

// Mistake in the team output
void fail(string msg)
{
	cout << "Validation failed: " << msg << endl;
	exit(43);
}

// Validation succeeded
void succ()
{
	cerr << "Validation successfull." << endl;
	exit(42);
}

const int maxd = 110;
const int maxc = 1010;
const int maxu = 610;

struct beer {
	string name;
	int perc;
	int unit6; // 1/6th units of alcohol
	int cost; // cents
};

int readfrac(int dec)
{
	int i,f;
	char c;
	int frac = 1;
	for(int i=0; i<dec; i++) frac *= 10;
	in >> i >> c >> f;
	return frac*i+f;
}

string printfrac(int x, int dec=0)
{
	char s[100];
	int frac = 1;
	for(int i=0; i<dec; i++) frac *= 10;
	sprintf(s,"%d.%0*d",x/frac,dec,x%frac);
	return string(s);
}

int main(int argc, char **argv)
{
	if ( argc<4 ) error("need at least 3 arguments");

	in.open(argv[1]);
	ans.open(argv[2]);
	feedbackdir = argv[3];

	// Read all team output into a stringstream to be able to feed it
	// both to libchecktestdata and then inspect it ourselves.
	string teamstr;
	char buf[1025];
	do {
		cin.read(buf,1024);
		buf[cin.gcount()] = 0;
		teamstr += buf;
	} while ( cin );
	team.str(teamstr);

	prog.str(output_grammar);
	init_checktestdata(prog,opt_whitespace_ok);

	if ( !checksyntax(team) ) fail("syntactic error in team output");

	if ( !team.seekg(0) ) error("could not reset team input stream");

	// First check for IMPOSSIBLE
	string anspos, teampos;
	if ( !(ans  >> anspos)  ) error("could not read from answer file");
	if ( !(team >> teampos) ) error("could not read from team output");

	if ( anspos=="IMPOSSIBLE" && teampos!="IMPOSSIBLE" )
		fail("answer has IMPOSSIBLE, but team does not");

	if ( anspos!="IMPOSSIBLE" && teampos=="IMPOSSIBLE" )
		fail("team has IMPOSSIBLE, but answer does not");

	if ( anspos=="IMPOSSIBLE" && teampos=="IMPOSSIBLE" ) succ();

	if ( !team.seekg(0) ) error("could not reset team input stream");

	// Read testdata input; m in cents, u in 1/10ths.
	int m, u, d;
	m = readfrac(2);
	u = readfrac(1);
	in >> d;

	map<string,beer> beers;

	string tmp;
	for(int i=0; i<d; i++) {
		int frac;
		char c;
		in >> tmp;
		beers[tmp].name = tmp;
		in >> beers[tmp].perc >> c >> c >> frac;
		beers[tmp].unit6 = beers[tmp].perc*6/frac;
		beers[tmp].cost = readfrac(2);
	}

	int m_used = 0, u6_used = 0;
	int n;
	int line = 0;
	map<string,bool> used;
	string name;
	while ( team >> name >> n ) {
		line++;
		ostringstream msg;
		msg << "on line " << line << ": ";
		if ( beers.count(name)==0 ) fail(msg.str()+"beer name '"+name+"' nonexistent");
		if ( used[name] ) fail(msg.str()+"beer '"+name+"' already used");
		if ( n==0 ) fail(msg.str()+"specified beer without using it (zero quantity)");
		m_used += n*beers[name].cost;
		u6_used += n*beers[name].unit6;
		used[name] = true;
	}

	if ( m_used!=m )
		fail("money spent "+printfrac(m_used,2)+" != "+printfrac(m,2)+" available");

	if ( u6_used%3!=0 )
		fail("units drunk "+printfrac(u6_used/6)+' '+printfrac(u6_used%6)+"/6 != "+
		     printfrac(u,1)+" aimed for");

	int u_used = (u6_used/3)*5;
	if ( u_used!=u )
		fail("units drunk "+printfrac(u_used,1)+" != "+printfrac(u,1)+" available");

	succ();
}
