#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>
#include <cmath>
#include <set>
using namespace std;
typedef long long ll;

struct vec2 {ll x,y; vec2 operator+(vec2 const a)const{return {x+a.x,y+a.y};}};
ll dot(vec2 a,vec2 b){return a.x*b.x+a.y*b.y;}
double clamp(double x,double a,double b){return max(min(x,b),a);}

bool contains_zero(vec2 *v,vec2 *end){
  bool does=false;
  for (auto i=v,j=end-1; i!=end; j=i++){
    does^=(i->y>=0)!=(j->y>=0) and i->x*abs(j->y-i->y)+(j->x-i->x)*abs(0-i->y)<=0;
  }
  return does;
}

ll area(vec2 *v,vec2 *end){
  ll res=0;
  for (auto i=v,j=end-1; i!=end; j=i++){
    res+=i->y*j->x-i->x*j->y;
  }
  return abs(res);
}

int main(){
  ios::sync_with_stdio(false);

  int n; cin>>n;
  double res=1e60L;
  bool inside_any=false;

  // height at (0,0)
  pair<double,double> origin={1e60L,1e60L};

  // height outside largest contour
  pair<ll,double> biggest={-1ll,1e60L};

  // set of all points for sanity-checking
  set<pair<int,int>> all;

  for (int i=0; i<n; i++){
    int h0,h1,m; cin>>h0>>h1>>m;
    static vec2 v[1000000];
    for (int j=m; j--;){
      cin>>v[j].x>>v[j].y;
      if (not all.insert({v[j].x,v[j].y}).second){
        cerr<<"ERROR: "<<v[j].x<<" "<<v[j].y<<" read more than once"<<endl;
        assert(false);
      }
    }
    biggest=max(biggest,{area(v,v+m),h0});
    bool contained=contains_zero(v,v+m);
    for (int j=0,k=m-1; j<m; k=j++){
      vec2 a=v[j],b=v[k],w={b.x-a.x,b.y-a.y};
      double t=clamp(-1.L*dot(a,w)/(double)dot(w,w),0,1);
      double cdist=hypot(a.x+w.x*t,a.y+w.y*t);
      res=min(res,(double)hypot(cdist,min(h0,h1)));
      if (contained) origin=min(origin,{cdist,h1});
    }
    inside_any|=contained;
  }

  if (inside_any){
    res=min(res,origin.second);
  }else{
    res=min(res,biggest.second);
  }

  cout.precision(10);
  cout<<fixed<<res<<endl;
}
