#include <algorithm>
#include <iostream>
#include <vector>
#include <cmath>
using namespace std;
double const inf=1e70L;
#define MAXN 12

int dx[4]={-1,-1, 1, 1};
int dy[4]={-1, 1, 1,-1};

struct Lamp{
    int x,y;
    int energy;
};

bool intersects(Lamp a,Lamp b,Lamp mid){
  a.x-=mid.x, a.y-=mid.y;
  b.x-=mid.x, b.y-=mid.y;

  for (int rot=4; rot--; swap(a.x,a.y*=-1), swap(b.x,b.y*=-1)){
    if ((a.y>=1)!=(b.y>=1)){
      // if |ax + (bx-ax)(1-ay)/(by-ay)| < 1 = goes through edge (corners not checked)
      if (abs(a.x*abs(b.y-a.y) + (b.x-a.x)*abs(1-a.y)) < abs(b.y-a.y)){
        return true;
      }
    }
    if ((a.y>=0)!=(b.y>=0)){
      // if |ax + (bx-ax)(0-ay)/(by-ay)| < 1 = goes through centre (to catch corners)
      if (abs(a.x*abs(b.y-a.y) + (b.x-a.x)*abs(0-a.y)) < abs(b.y-a.y)){
        return true;
      }
    }
  }
  return false;
}

bool passes(Lamp a,Lamp b,Lamp mid){
  a.x-=mid.x, a.y-=mid.y;
  b.x-=mid.x, b.y-=mid.y;
  if ((a.y>=0)==(b.y>=0)) return false;

  // if ax + (bx-ax)(0-ay)/(by-ay) < 0
  if (a.x*abs(b.y-a.y) + (b.x-a.x)*abs(0-a.y) < 0){
    return true;
  }
  return false;
}

double dp[MAXN*4][1<<MAXN];
pair<double,int> dist[MAXN*4][MAXN*4];

int main(){
  int n; cin>>n;
  vector<Lamp> lamp(n);
  for (auto &i: lamp) cin>>i.x>>i.y>>i.energy;

  // get point-to-point distances and transitions
  for (int i=0; i<n; i++) for (int iz=0; iz<4; iz++){
    for (int j=0; j<n; j++) for (int jz=0; jz<4; jz++){
      dist[i*4+iz][j*4+jz].first=hypotl(
        (lamp[i].x+dx[iz])-(lamp[j].x+dx[jz]),
        (lamp[i].y+dy[iz])-(lamp[j].y+dy[jz]));
      for (int k=0; k<n; k++){
        if (intersects({lamp[i].x+dx[iz],lamp[i].y+dy[iz]},
                       {lamp[j].x+dx[jz],lamp[j].y+dy[jz]}, lamp[k])){
          dist[i*4+iz][j*4+jz].first=inf;
        }
        else if (passes({lamp[i].x+dx[iz],lamp[i].y+dy[iz]},
                        {lamp[j].x+dx[jz],lamp[j].y+dy[jz]}, lamp[k])){
            dist[i*4+iz][j*4+jz].second|=1<<k;
        }
      }
    }
  }

  double res=inf;

  // try every point as a start
  for (int sp=0; sp<n*4; sp++){
    // initialise dp table
    for (int i=n*4; i--;)
      for (int j=1<<n; j--;)
          dp[i][j]=inf;
    dp[sp][0]=0;

    // compute shortest paths
    for (int rep=n*4; rep-->sp;){
      bool changed=false;
      for (int x=n*4; x-->sp;){
        for (int mask=1<<n; mask--;) if (dp[x][mask]<res) {
          for (int y=n*4; y-->sp;){
            if (dist[x][y].first<inf){
              if (dp[y][mask^dist[x][y].second] > dp[x][mask]+dist[x][y].first + 1e-10L){
                dp[y][mask^dist[x][y].second] = dp[x][mask]+dist[x][y].first;
                changed=true;
              }
            }
          }
        }
      }
      if (not changed) break;
    }

    // compute answer
    for (int mask=1<<n; mask--;){
      int value=0;
      for (int i=n; i--;) value+=lamp[i].energy*(mask&(1<<i)?1:-1);
      if (not value) res=min(res,dp[sp][mask]);
    }
  }

  // print answer
  if (res<inf){
    cout.precision(10);
    cout<<fixed<<res<<endl;
  }else{
    cout<<"IMPOSSIBLE"<<endl;
  }
}
