#!/bin/bash

shopt -s nullglob

if [ "$#" -ne 0 ]
  then
    problems="$@"
  else
    problems="$(for i in ./problems/* ; do basename "$i" ; done)"
  fi

for problem in $problems
  do
    checker="diff -wB"
    if [ -e ./problems/"$problem"/data/checker ]
      then
        checker=./problems/"$problem"/data/checker
      fi

    if [[ ! -z $problem ]] && [ -e ./problems/"$problem" ]
      then
        printf 'Checking problem "%s"\n' "$problem"
      else
        printf 'Error: no problem exists named "%s"\n' "$problem" >&2
        exit 1
      fi

    type checktestdata >/dev/null 2>&1 && \
    for type in "sample" "secret"
      do
        for tf in ./problems/"$problem"/data/"$type"/*.in
          do
            checktestdata ./problems/"$problem"/input_format_validators/*.ctd $tf
          done
      done

    for codefile in ./problems/"$problem"/submissions/accepted/*.*
      do
        executable=./problems/"$problem"/a.out
        friendly="`basename "$codefile"`"
        ext="${codefile##*.}"

        tests=0; ac=0;

        if [ "$ext" = "py" -o "$ext" = "py2" -o "$ext" = "py3" ]
          then
            cp "$codefile" "$executable"
            chmod "+x" "$executable"
        elif [ "$ext" = "java" -o "$ext" = "class" ]
          then
            printf ' skipping "%s"\n' "$friendly"
            continue
          else
            g++ -std=c++11 -O2 -o "$executable" "$codefile"
            if [ $? -ne 0 ]
              then
                printf 'Failed to compile %s\n' "$friendly" >&2
                continue
              fi
          fi

        echo -n '  '
        for type in "sample" "secret"
          do
            for tf in ./problems/"$problem"/data/"$type"/*.in
              do
                tn="`basename "$tf" .in`"
                answer="problems/"$problem"/data/"$type"/"$tn".ans"
                if [ -f "$answer" ]
                  then
                    checker_output="`$checker <("$executable" < "$tf" 2>/dev/null) "$answer"`"
                    if [ -z "$checker_output" ] || [ "$checker_output" = "OK" ]
                      then
                        echo -n '+' && ((ac++))
                      else
                        printf '\nTest %s %s:\n%s\n' "$type" "$tn" "$checker_output" >&2
                        echo -n '-'
                      fi
                    ((tests++))
                  else
                    output="$("$executable" < "$tf" 2>/dev/null)"
                    echo -n '?'
                  fi
              done
          done
        printf '%3d/%d %s\n' "$ac" "$tests" "$friendly"
        rm -f $executable

        if [ $tests == 0 ]
          then
            echo "No tests"
            break
          fi
      done
    echo
  done

exit 0
