#!/bin/bash

shopt -s nullglob

if [ "$#" -ne 0 ]
  then
    problems="$@"
  else
    problems="$(for i in ./problems/* ; do basename "$i" ; done)"
  fi

for problem in $problems
  do
    # Clear any existing data
    rm -rf ./packages/"$problem"
    mkdir -p ./packages/"$problem"

    cp -r ./problems/"$problem"/* ./packages/"$problem"/

    # Create metadata
    (
      echo "probid = $problem"; \
      echo -n "name = "; \
      head -n1 ./problems/"$problem"/problem_statement/problem.en.tex \
        | awk -F'[{}]' '{print $2}'; \
    ) > ./packages/"$problem"/domjudge-problem.ini

    # Add problem statement
    cp ./problems/"$problem"/problem_statement/problem.en.tex ./packages/"$problem"/problem.txt

    # Generate new archive
    rm -f ./packages/"$problem.zip"
    ( cd ./packages/"$problem" && zip -r ../"$problem.zip" ./* )
  done

exit 0
